const socketIO = require('socket.io');
require('./libs/object.polyfills');
const PacketHandler = require('./libs/server/parse.packethandler');
const Room = require('./libs/server/parse.room');

module.exports = function(opts) {
	this.initialize = function() {
		this.loglevel = (opts.hasOwnProperty('loglevel')) ? opts.loglevel : 'default';
		this.io = socketIO(opts.http);
		this.clients = {};
		this.rooms = {};
		this.generalRoomID = this.createRoom({ name:'general', creator:'server' });
		this.updateStats();
		this.setupListeners();
	}

	this.setupListeners = function() {
		this.packethandler = PacketHandler(this);
	}

	this.broadcast = function(socket, evt, data) {
		socket.broadcast.emit(evt, data);
	}

	this.broadcastAll = function(evt, data) {
		this.io.emit(evt, data);
	}

	this.updateStats = function() {
		process.title = 'Parse | online: ' + this.clients.length() + ' | rooms: ' + this.rooms.length();
	}

	this.createRoom = function(roomData) {
		if(!this.rooms.containsKey(roomData.name)) {
			roomData.parse = this;
			var room = new Room(roomData);
			room.id = this.rooms.push(room);
			this.broadcastAll('create_room', this.getRoomInfo(room.id));
			if(this.loglevel === 'debug') console.log('Room \'' + room.name + '\' created.');
			this.updateStats();
			return room.id;
		}
	}

	this.deleteRoom = function(roomID) {
		if(this.rooms.containsKey(roomID)) {
			var room = this.rooms[roomID];
			this.rooms.remove(room);
			this.broadcastAll('delete_room', room);
			if(this.loglevel === 'debug') console.log('Room \'' + room.name + '\' deleted.');
			this.updateStats();
		}
	}

	this.joinRoom = function(socket, roomID) {
		if(this.clients.containsKey(socket.id) && this.rooms.containsKey(roomID)) {
			var client = this.clients[socket.id];
			this.clients[socket.id].currentRoomID = roomID;
			var room = this.rooms[roomID];
			room.join(this.clients[socket.id], socket);
			socket.emit('join_room', this.parseRoomInfo(room));
			this.broadcast(socket, 'room_connect', { client: this.parseClientInfo(client), room: this.parseRoomInfo(room) });
		}
	}

	this.leaveRoom = function(socket, roomID) {
		if(this.clients.containsKey(socket.id) && this.rooms.containsKey(roomID)) {
			var client = this.clients[socket.id];
			var room = this.rooms[roomID];
			room.leave(client, socket);
			socket.emit('leave_room', this.parseRoomInfo(room));
			this.broadcast(socket, 'room_disconnect', { client: this.parseClientInfo(client), room: this.parseRoomInfo(room) });
		}
	}

	this.getServerInfo = function() {
		var data = { online: this.clients.length(), rooms: {}, clients: {} };
		this.rooms.forEach(room => {
			data.rooms[room.id] = this.parseRoomInfo(room);
		});
		this.clients.forEach(client => {
			data.clients[client.id] = this.parseClientInfo(client);
		});
		return data;
	}

	this.getRoomInfo = function(roomID) {
		if(this.rooms.containsKey(roomID)) {
			var room = this.rooms[roomID];
			return this.parseRoomInfo(room);
		}
	}

	this.parseClientInfo = function(client) {
		return { id: client.id, username: client.username };
	}

	this.parseRoomInfo = function(room) {
		var online = room.clients.length();
		var roomClients = {};
		room.clients.forEach(client => {
			roomClients[client.id] = this.parseClientInfo(client);
		});
		return { id: room.id, name: room.name, creator: room.creator, online, clients: roomClients };
	}

	this.sendPacket = function(socket, packetData) {
		if(this.clients.containsKey(socket.id)) {
			var client = this.clients[socket.id];
			if(packetData.inRoom) {
				if(this.rooms.containsKey(client.currentRoomID)) {
					var room = this.rooms[client.currentRoomID];
					room.broadcast(socket, 'receive_packet', JSON.stringify(packetData));
				} else {
					console.log('room doesn\'t exist');
				}
			} else {
				this.broadcast(socket, 'receive_packet', JSON.stringify(packetData));
			}
		}
	}

	this.initialize();
	return this.io;
}